class Report < ActiveRecord::Base
  self.primary_key = :id
  belongs_to :connect
  has_many :requirements, dependent: :destroy

  validates :name, uniqueness: {message: "报表名称已存在!"}

  MAPTYPE = {
      "点" => 0,
      "线" => 5,
      "面" => 10
  }

  def report_date(params)
    results = {charts: [], category_names: []}
    sql_code = self.sql_code
    self.requirements.each do |r|
      if r.is_null && params[r.sql_key].blank?
        sql_code = sql_code.gsub(r.sql_default, r.sql_replace)
      else
        sql_code = sql_code.gsub("${#{r.sql_key}}", "#{params[r.sql_key.to_sym]}" || "")
      end
    end
    sql_results = self.connect.exec(sql_code).clone
    series = eval(self.series || "{}")
    category_names = []
    series.each do |k, v|
      data = []
      sql_results.each do |r|
        rown = {}
        r.each do |k1, v1|
          rown[k1.downcase] = v1
        end
        category_name = self.category_name.blank? ? "" : self.get_category_name(rown[self.category_name.downcase])
        data << {category_name: category_name, value: rown[v].to_i}
        category_names << category_name
      end
      results[:charts] << {series_name: k, data: data}
    end

    flip_table = {thead: series.collect { |k, v| k }, tbody: []}
    sql_results.each do |r|
      tr = []
      rown = {}
      r.each do |k1, v1|
        rown[k1.downcase] = v1
      end
      category_name = self.category_name.blank? ? "" : self.get_category_name(rown[self.category_name.downcase])
      tr << category_name
      series.each do |k, v|
        tr << rown[v].to_i
      end
      flip_table[:tbody] << tr
    end
    results[:flip_table] = flip_table
    results[:category_names] = category_names.uniq()
    results
  end

  def get_category_name(category_name)
    name = "#{category_name}"
    if (self.is_sql_category==0 || !self.is_sql_category)
      dict = eval(self.dictionary.blank? ? "{}" : self.dictionary)
      name = dict[name.to_sym] if dict[name.to_sym]
    end
    name
  end
end